/**
 * @file gt_720f.h
 * @author CuBeatSystems
 * @author Shinichiro Nakamura
 * @brief GT-720F device driver (Version 0.0.1)
 * @details
 * Reference:
 * Application Note AN0003
 * Binary Messages Of SkyTraq Venus 5 GPS Receiver
 * Ver 0.3 Feb. 1, 2007
 * SkyTraq Technology, Inc.
 */

/*
 *  TOPPERS Realtime System Sample Program - LPCXpresso GPS Clock
 *
 *  Copyright (C) 2010-2013 CuBeatSystems
 *  Copyright (C) 2010-2013 Shinichiro Nakamura
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 */

#ifndef GT_720F_H
#define GT_720F_H

/**
 * @brief GT-720Fハンドラの実装。
 */
typedef struct {
    /**
     * @brief リード関数。
     *
     * @param buf 読み込み対象データ。
     * @param siz 読み込みデータバイト数。
     *
     * @retval 0 成功。
     * @retval !0 失敗。
     */
    int (*func_read)(unsigned char *buf, int siz);

    /**
     * @brief ライト関数。
     *
     * @param buf 書き込み対象データ。
     * @param siz 書き込みデータバイト数。
     *
     * @retval 0 成功。
     * @retval !0 失敗。
     */
    int (*func_write)(unsigned char *buf, int siz);
} gt_720f_t;

/**
 * @brief セットアップを実行する。
 *
 * @param p ハンドラ。
 * @param WRITE ライト関数。
 * @param READ リード関数。
 */
#define GT_720F_SETUP(P, WRITE, READ) \
    do { \
        (P)->func_write = (WRITE); \
        (P)->func_read = (READ); \
    } while (0)

/**
 * @brief Start mode.
 */
typedef enum {
    Gt720fStartModeNochange = 0,
    Gt720fStartModeHotStart = 1,
    Gt720fStartModeWarmStart = 2,
    Gt720fStartModeColdStart = 3,
    Gt720fStartModeTest = 4,
} Gt720fStartMode;

/**
 * @brief SYSTEM RESTART - Force system to restart (0x1)
 *
 * @details
 * This is a request message which will reset and restart the GPS receiver.
 * This command is issued from the host to GPS receiver and GPS receiver
 * should respond with an ACK or NACK.
 *
 * The payload length is 15 bytes.
 */
int gt_720f_system_restart(
        gt_720f_t *p,
        Gt720fStartMode start_mode,
        unsigned short year,
        unsigned char month,
        unsigned char day,
        unsigned char hour,
        unsigned char minute,
        unsigned char second,
        short latitude,
        short longitude,
        short altitude);

/**
 * @brief QUERY SOFTWARE VERSION
 *
 * @details
 * Query revision information of loaded software (0x2)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to retrieve loaded software version. The GPS receiver should respond
 * with an ACK along with information on software version when succeeded
 * and should respond with an NACK when failed.
 *
 * The payload length is 2 bytes.
 */
int gt_720f_query_software_version(gt_720f_t *p);

/**
 * @brief QUERY SOFTWARE CRC
 *
 * @details
 * Query CRC information of loaded software (0x3)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to retrieve loaded software CRC. The GPS receiver should respond with an
 * ACK along with information on software version when succeeded and should
 * respond with an NACK when failed.
 *
 * The payload length is 2 bytes.
 */
int gt_720f_query_software_crc(gt_720f_t *p);

/**
 * SET FACTORY DEFAULTS - Set the system to factory default values (0x4)
 *
 * This is a request message which is issued from the host to GPS receiver.
 * It will reset the GPS receiver's internal parameters to factory default values.
 * The GPS receiver should respond with an ACK when succeeded and should
 * respond with an NACK when failed.
 * The user data will be erased and filled with factory default values.
 * The payload length is 2 bytes.
 */
int gt_720f_set_factory_defaults(gt_720f_t *p);

typedef enum {
    Gt720fBaudRate4800 = 0,
    Gt720fBaudRate9600 = 1,
    Gt720fBaudRate19200 = 2,
    Gt720fBaudRate38400 = 3,
    Gt720fBaudRate57600 = 4,
    Gt720fBaudRate11520 = 5,
} Gt720fBaudRate;

/**
 * @brief CONFIGURE SERIAL PORT
 *
 * @details
 * Set up serial port property (0x5)
 *
 * This is a request message which will configure the serial COM port,
 * baud rate. This command is issued from the host to GPS receiver and GPS
 * receiver should respond with an ACK or NACK.
 *
 * The payload length is 4 bytes.
 */
int gt_720f_configure_serial_port(
        gt_720f_t *p,
        Gt720fBaudRate baud_rate);

/**
 * @brief CONFIGURE NMEA MESSAGE
 *
 * @details
 * Configure NMEA message interval (0x8)
 *
 * This is a request message which will set NMEA message configuration.
 * This command is issued from the host to GPS receiver and GPS receiver
 * should respond with an ACK or NACK.
 *
 * The payload length is 9 bytes.
 */
int gt_720f_configure_nmea_message_interval(
        gt_720f_t *p,
        unsigned char gga,
        unsigned char gsa,
        unsigned char gsv,
        unsigned char gll,
        unsigned char rmc,
        unsigned char vtg,
        unsigned char zda);

/**
 * @brief メッセージタイプ。
 */
typedef enum {
    MessageTypeNoOutput = 0,    /**< 出力しない。 */
    MessageTypeNMEA = 1,        /**< NMEA。 */
    MessageTypeBinary = 2,      /**< バイナリ。 */
} MessageType;

/**
 * @brief CONFIGURE MESSAGE TYPE
 *
 * @details
 * Configure and select output message type (0x9)
 *
 * This is a request message which will change the GPS receiver output
 * message type. This command is issued from the host to GPS receiver and
 * GPS receiver should respond with an ACK or NACK.
 *
 * The payload length is 3 bytes.
 */
int gt_720f_configure_message_type(
        gt_720f_t *p,
        MessageType type);

/**
 * @brief CONFIGURE SYSTEM POWER MODE
 *
 * @details
 * Set the power mode of GPS system (0xC)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to configure the system power mode. The GSP receiver should respond with
 * an ACK when succeeded and should respond with an NACK when failed.
 *
 * The payload length is 3 bytes.
 */

/**
 * @brief CONFIGURE SYSTEM POSITION RATE
 *
 * @details
 * Configure the position update rate of GPS system (0xE)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to configure the system position update rate.
 * Receivers with position rate 4 or higher needs to configure baud rate to
 * 38400 or higher value.The GSP receiver should respond with an ACK when
 * succeeded and should respond with an NACK when failed.
 *
 * The payload length is 3 bytes.
 */

/**
 * @brief QUERY POSITION UPDATE RATE
 *
 * @details
 * Query the position update rate of GPS system (0x10)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to query position update rate. The GSP receiver should respond with an
 * ACK along with information on software version when succeeded and should
 * respond with an NACK when failed.
 *
 * The payload length is 1 byte.
 */

/**
 * @brief Get Ephemeris
 *
 * @details
 * Get ephemeris used of firmware (0x30)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to retrieve ephemeris data. The GSP receiver should respond with an ACK
 * along with information on ephemeris when succeeded and should respond
 * with an NACK when failed.
 *
 * The payload length is 2 bytes.
 */

/**
 * @brief Set Ephemeris
 *
 * @details
 * Set ephemeris to GPS firmware (0x31)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to set ephemeris data (open an ephemeris file) to GPS receiver.
 * The GSP receiver should respond with an ACK when succeeded and should
 * respond with an NACK when failed.
 *
 * The payload length is 87 bytes.
 */

/**
 * @brief CONFIGURE WAAS
 *
 * @details
 * Configure the enable or disable of WAAS (0x37)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to enable or disable WAAS. The GSP receiver should respond with an ACK
 * when succeeded and should respond with an NACK when failed.
 *
 * The payload length is 3 bytes.
 */

/**
 * @brief QUERY WAAS STATUS
 *
 * @details
 * Query WAAS status of GPS receiver (0x38)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to query WAAS status. The GSP receiver should respond with an ACK along
 * with AGPS aiding status when succeeded and should respond with an NACK
 * when failed.
 *
 * The payload length is 1 bytes.
 */

/**
 * @brief CONFIGURE NAVIGATION MODE
 *
 * @details
 * Configure the navigation mode of GPS system (0x3c)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to configure the system navigation mode.
 * The GSP receiver should respond with an ACK when succeeded and should
 * respond with an NACK when failed.
 *
 * The payload length is 3 bytes.
 */

/**
 * @brief QUERY NAVIGATION MODE
 *
 * @details
 * Query the navigation mode of GPS receiver (0x3d)
 *
 * This is a request message which is issued from the host to GPS receiver
 * to query navigation mode.
 * The GSP receiver should respond with an ACK along with navigation mode
 * when succeeded and should respond with an NACK when failed.
 *
 * The payload length is 1 bytes.
 */

#endif

