/**
 * @file nmea.c
 * @author CuBeatSystems
 * @author Shinichiro Nakamura
 * @brief NMEA parser (Version 0.0.1)
 */

/*
 *  TOPPERS Realtime System Sample Program - LPCXpresso GPS Clock
 *
 *  Copyright (C) 2010-2013 CuBeatSystems
 *  Copyright (C) 2010-2013 Shinichiro Nakamura
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 */

#include "nmea.h"

#define MAX_ARGS    (24)
#define MAX_CHAR    (16)

#define ABORT_GETC(N)   (N < 0)
#define ABORT_CBFUNC(N) (N < 0)

/**
 * @brief 与えられた16進数文字列を値にして返す。
 * @details 2桁の16進数文字列以外を与えた場合、不正な結果を返す。
 * @param s 2桁の16進数文字列。
 * @return 値。
 */
static int get_hexnum(const char *s)
{
    static const char *numidx = "0123456789ABCDEF";
    int a = -1, b = -1;
    char *p = (char *)numidx;
    int idx = 0;
    while (*p) {
        if (s[0] == *p) {
            a = idx;
        }
        if (s[1] == *p) {
            b = idx;
        }
        p++;
        idx++;
    }
    if ((a < 0) || (b < 0)) {
        return -1;
    }
    return (a * 16) + b;
}

/**
 * @brief NMEAセンテンスのパースを実行する。
 * @details この関数はnmea_getcかnmea_cbfuncが負値を返すまで処理を返さない。
 *
 * @param nmea_getc NMEA受信関数。
 *                  この関数が負値を返すと、その値をこの関数が返す。
 * @param nmea_cbfunc １センテンス受信した時に呼ばれるコールバック関数。
 *                    この関数が負値を返すと、その値をこの関数が返す。
 *
 * @return nmea_getcあるいは、nmea_cbfuncが返した値。
 */
int nmea_parse(
        int (*nmea_getc)(void),
        int (*nmea_cbfunc)(int argc, char **argv, void *extobj),
        void *extobj)
{
    char args[MAX_ARGS][MAX_CHAR];
    int pcnt = 0;
    int ccnt = 0;
    unsigned char csum = 0;
    int c = 0;
    do {
        pcnt = 0;
        ccnt = 0;
        csum = 0;

        /*
         * Waiting a header.
         */
        do {
            c = nmea_getc();
            if (ABORT_GETC(c)) {
                return c;
            }
            args[pcnt][ccnt] = c;
        } while (c != '$');
        ccnt++;

        /*
         * Waiting the end of the body.
         */
        do {
            c = nmea_getc();
            if (ABORT_GETC(c)) {
                return c;
            }
            if (c != '*') {
                csum ^= c;
            }
            if ((c == ',') || (c == '*')) {
                args[pcnt][ccnt] = '\0';
                pcnt++;
                ccnt = 0;
            } else {
                args[pcnt][ccnt] = c;
                ccnt++;
            }
        } while (c != '*');

        /*
         * Waiting the check sum.
         */
        do {
            c = nmea_getc();
            if (ABORT_GETC(c)) {
                return c;
            }
            if ((c == '\r') || (c == '\n')) {
                args[pcnt][ccnt] = '\0';
                pcnt++;
                ccnt = 0;
            } else {
                args[pcnt][ccnt] = c;
                ccnt++;
            }
        } while ((c != '\r') && (c != '\n'));

        /*
         * Callback the callback.
         */
        {
            int i;
            char *p[MAX_ARGS];
            for (i = 0; i < MAX_ARGS; i++) {
                p[i] = args[i];
            }
            /*
             * 最後のフィールドはチェックサム。
             * チェックサム検証が通ればコールバックを呼ぶ。
             */
            if (get_hexnum(args[pcnt - 1]) == csum) {
                int r = nmea_cbfunc(pcnt, p, extobj);
                if (ABORT_CBFUNC(r)) {
                    return r;
                }
            }
        }
    } while (1);

    return 0;
}

