/**
 * @file ntshell_usrcmd.c
 * @author CuBeatSystems
 * @author Shinichiro Nakamura
 */

/*
 *  TOPPERS Realtime System Sample Program - LPCXpresso GPS Clock
 *
 *  Copyright (C) 2010-2013 CuBeatSystems
 *  Copyright (C) 2010-2013 Shinichiro Nakamura
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 */

#include <kernel.h>
#include <target_syssvc.h>
#include <syssvc/serial.h>
#include "kernel_cfg.h"

#include "ntopt.h"
#include "ntlibc.h"
#include "ntshell_config.h"

#include "task_display.h"

static int usrcmd_ntopt_callback(int argc, char **argv, void *extobj);
static int usrcmd_help(int argc, char **argv);
static int usrcmd_info(int argc, char **argv);
static int usrcmd_display(int argc, char **argv);

/**
 * @brief コマンドテーブル。
 */
typedef struct {
    char *cmd;  /**< コマンド名。 */
    char *desc; /**< コマンド説明。 */
    int (*func)(int argc, char **argv); /**< コールバック関数。 */
} cmd_table_t;

/**
 * @brief コマンドの定義。
 * @details システムで必要なコマンドの実装を追加すると良い。
 */
static const cmd_table_t cmdlist[] = {
    { "help", "Help.", usrcmd_help },
    { "info", "Information.", usrcmd_info },
    { "display", "Command for display task.", usrcmd_display },
    { NULL, NULL, NULL }
};

/**
 * @brief NT-Shellコマンドを実行する。
 * @details
 *
 * @param text テキスト。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
int ntshell_usrcmd_execute(const char *text)
{
    /*
     * ntoptライブラリを使って引数を解析し実行する。
     */
    return ntopt_parse(text, usrcmd_ntopt_callback, NULL);
}

/**
 * @brief ユーザコマンドを実行する。
 * @details
 *
 * @param argc 引数の数。
 * @param argv 引数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
static int usrcmd_ntopt_callback(int argc, char **argv, void *extobj)
{
    /*
     * 引数の数が0ならば何もしない。
     * 単にenterが押されただけと解釈する。
     */
    if (argc == 0) {
        return 0;
    }

    /*
     * コマンドテーブルを探索して、
     * コマンド名が一致したらコールバック関数を呼び出す。
     */
    const cmd_table_t *p = &cmdlist[0];
    while (p->cmd != NULL) {
        if (ntlibc_strcmp((const char *)argv[0], p->cmd) == 0) {
            return p->func(argc, argv);
        }
        p++;
    }

    /*
     * ここに到達するということは、未知のコマンドである。
     */
    {
        static const char *msg = "Unknown command. (help: display help.)\n";
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)msg, ntlibc_strlen(msg));
    }
    return 0;
}

/**
 * @brief helpコマンド。
 * @details
 *
 * @param argc 引数の数。
 * @param argv 引数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
static int usrcmd_help(int argc, char **argv)
{
    const cmd_table_t *p = &cmdlist[0];
    char buf[128];

    /*
     * コマンド名とコマンド説明を列挙する。
     */
    while (p->cmd != NULL) {
        ntlibc_strcpy(buf, p->cmd);
        ntlibc_strcat(buf, "\t:");
        ntlibc_strcat(buf, p->desc);
        ntlibc_strcat(buf, "\n");
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)buf, ntlibc_strlen(buf));
        tslp_tsk(5);
        p++;
    }
    return 0;
}

/**
 * @brief infoコマンド。
 * @details
 *
 * @param argc 引数の数。
 * @param argv 引数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
static int usrcmd_info(int argc, char **argv)
{
    if (argc != 2) {
        static const char *msg = "info sys\ninfo ver\n";
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)msg, ntlibc_strlen(msg) + 1);
        return 0;
    }
    if (ntlibc_strcmp(argv[1], "sys") == 0) {
        static const char *msg = "LPCXpressoClock\n";
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)msg, ntlibc_strlen(msg) + 1);
        return 0;
    }
    if (ntlibc_strcmp(argv[1], "ver") == 0) {
        static const char *msg = "0.0.1\n";
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)msg, ntlibc_strlen(msg) + 1);
        return 0;
    }
    {
        static const char *msg = "Unknown sub command.\n";
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)msg, ntlibc_strlen(msg) + 1);
    }
    return -1;
}

/**
 * @brief displayコマンド。
 * @details
 *
 * @param argc 引数の数。
 * @param argv 引数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
static int usrcmd_display(int argc, char **argv)
{
    if (argc != 3) {
        static const char *msg = "display text [TTTTTT]\n";
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)msg, ntlibc_strlen(msg) + 1);
        return 0;
    }
    if (ntlibc_strcmp(argv[1], "text") == 0) {
        int i;
        int len = ntlibc_strlen(argv[2]);
        uint8_t segsel[6];
        for (i = 0; i < sizeof(segsel); i++) {
            segsel[i] = 0;
        }
        for (i = 0; i < len; i++) {
            switch (i) {
                case 0:
                case 1:
                case 2:
                case 3:
                case 4:
                case 5:
                    /*
                     * ７セグ。
                     */
                    segsel[i] = argv[2][i];
                    break;
                default:
                    /*
                     * 無視する。
                     */
                    break;
            }
        }
        tskapi_display_update(
                segsel[0],
                segsel[1],
                segsel[2],
                segsel[3],
                segsel[4],
                segsel[5],
                0);
        return 0;
    }
    {
        static const char *msg = "Unknown sub command.\n";
        serial_wri_dat(SHELL_PORTID,
                (const char_t *)msg, ntlibc_strlen(msg) + 1);
    }

    return 0;
}

