/**
 * @file task_display.c
 * @author CuBeatSystems
 * @author Shinichiro Nakamura
 */

/*
 *  TOPPERS Realtime System Sample Program - LPCXpresso GPS Clock
 *
 *  Copyright (C) 2010-2013 CuBeatSystems
 *  Copyright (C) 2010-2013 Shinichiro Nakamura
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 */

#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include <itron.h>
#include "kernel_cfg.h"
#include "display.h"
#include "task_display.h"

#define DISPLAY_CMD_UPDATE  (0) /**< 表示更新コマンド。 */

/**
 * @brief ディスプレイタスクへのメッセージ通信用構造体。
 */
typedef struct {
    T_MSG header;       /**< カーネルが使用するメッセージヘッダ。 */
    uint8_t cmd;        /**< コマンド。 */
    uint8_t param[64];  /**< 最大64バイトのパラメータ領域。 */
} display_msg_t;

/**
 * @brief ディスプレイタスクの更新コマンドに対するパラメータ。
 */
typedef struct {
    uint8_t h10;
    uint8_t h01;
    uint8_t m10;
    uint8_t m01;
    uint8_t s10;
    uint8_t s01;
    uint8_t col;
} display_param_update_t;

/**
 * @brief ワーク用変数。
 */
typedef struct {
    display_t display;
} display_work_t;

static display_work_t work; /**< ワーク用。 */

/**
 * @brief 更新コマンドの実行。
 * @param param パラメータ。
 */
void cmd_update(display_param_update_t *param)
{
    display_7seg_write(
            &(work.display),
            param->h10,
            param->h01,
            param->m10,
            param->m01,
            param->s10,
            param->s01,
            param->col);
}

/**
 * @brief Display task.
 */
void task_display(intptr_t exinf)
{
    syslog(LOG_NOTICE, "Display task initialized.");

    /*
     * ハードウェアをオープンする。
     */
    display_open(&(work.display));

    {
        /*
         * 輝度設定を初期化する。
         */
        int i;
        for (i = 0; i < 8; i++) {
            display_7seg_brightness(&(work.display), i, 7);
        }
        display_7seg_write(
                &(work.display),
                SEGSEL_G,
                SEGSEL_G,
                SEGSEL_G,
                SEGSEL_G,
                SEGSEL_G,
                SEGSEL_G,
                0);
    }

    display_msg_t *p;
    while (1) {
        /*
         * メールボックスの状態を観察し、
         * メッセージがあれば処理を実行する。
         */
        if (prcv_mbx(MBX_DISPLAY, (T_MSG **)&p) == E_OK) {
            uint8_t cmd = ((display_msg_t *)p)->cmd;
            void *param = ((display_msg_t *)p)->param;
            switch (cmd) {
                case DISPLAY_CMD_UPDATE:
                    cmd_update((display_param_update_t *)param);
                    break;
                default:
                    syslog(LOG_NOTICE, "Unknown command 0x%x.", cmd);
                    break;
            }
            rel_mpf(MPF_DISPLAY, (VP)p);
        }
        display_7seg_clock(&(work.display));
        tslp_tsk(2);
    }

    display_close(&(work.display));

    syslog(LOG_NOTICE, "Display task terminated.");
    ext_ker();
}

/**
 * @brief ディスプレイの更新を要求する。
 * @details この関数はディスプレイタスクにリクエストを要求したいタスクが呼び出す為のものである。
 */
void tskapi_display_update(
        uint8_t h10,
        uint8_t h01,
        uint8_t m10,
        uint8_t m01,
        uint8_t s10,
        uint8_t s01,
        uint8_t col)
{
    VP vp;

    /*
     * メモリプールからバッファを取得する。
     */
    get_mpf(MPF_DISPLAY, &vp);

    /*
     * コマンドとパラメータを設定する。
     */
    ((display_msg_t *)vp)->cmd = DISPLAY_CMD_UPDATE;
    display_param_update_t *param =
        (display_param_update_t *)&(((display_msg_t *)vp)->param);
    param->h10 = h10;
    param->h01 = h01;
    param->m10 = m10;
    param->m01 = m01;
    param->s10 = s10;
    param->s01 = s01;
    param->col = col;

    /*
     * メールボックスに送信する。
     */
    snd_mbx(MBX_DISPLAY, vp);
}

