/**
 * @file task_ntshell.c
 * @author CuBeatSystems
 * @author Shinichiro Nakamura
 */

/*
 *  TOPPERS Realtime System Sample Program - LPCXpresso GPS Clock
 *
 *  Copyright (C) 2010-2013 CuBeatSystems
 *  Copyright (C) 2010-2013 Shinichiro Nakamura
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 */

#include <kernel.h>
#include <target_syssvc.h>
#include <syssvc/serial.h>
#include "ntshell.h"
#include "ntshell_usrcmd.h"
#include "ntshell_config.h"

static int func_ntshell_read(char *buf, int cnt);
static int func_ntshell_write(const char *buf, int cnt);
static int func_ntshell_callback(const char *text, void *extobj);

/**
 * @brief NT-Shellライブラリ用のリード関数。
 * @details この関数は指定した読み込みバイト数に達するまでブロックする。
 *
 * @param buf バッファ。
 * @param cnt 読み込みバイト数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
static int func_ntshell_read(char *buf, int cnt)
{
    return serial_rea_dat(SHELL_PORTID, (char_t *)buf, cnt);
}

/**
 * @brief NT-Shellライブラリ用のライト関数。
 * @details この関数は指定した書き込みバイト数に達するまでブロックする。
 *
 * @param buf バッファ。
 * @param cnt 書き込みバイト数。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
static int func_ntshell_write(const char *buf, int cnt)
{
    return serial_wri_dat(SHELL_PORTID, (const char_t *)buf, cnt);
}

/**
 * @brief NT-Shellライブラリのコールバック関数。
 * @details
 *
 * @param text ユーザから入力された文字列。
 * @param extobj 拡張オブジェクト。
 *
 * @retval 0 成功。
 * @retval !0 失敗。
 */
static int func_ntshell_callback(const char *text, void *extobj)
{
    /*
     * ユーザコマンドを実行する。
     */
    return ntshell_usrcmd_execute(text);
}

/**
 * @brief NT-Shell Taskの本体。
 * @param exinf オプション。
 */
void task_ntshell(intptr_t exinf)
{
    ntshell_t ntshell;

    /*
     * シリアルポートをオープンする。
     */
    serial_opn_por(SHELL_PORTID);

    /*
     * 制御コードが端末からシステムに入力された時、
     * そのままエコーバックすると都合が悪い。
     * そこでエコーバックのオプションは外す。
     */
    serial_ctl_por(SHELL_PORTID, (IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV));

    /*
     * NT-Shellライブラリを実行する。
     */
    ntshell_init(
            &ntshell,
            func_ntshell_read,
            func_ntshell_write,
            func_ntshell_callback,
            NULL);
    ntshell_set_prompt(&ntshell, "GPSCLOCK>");
    while (1) {
        ntshell_execute(&ntshell);
    }

    /*
     * ここに到達することはない。
     */
    ext_tsk();
}

